#!/bin/python3

"""
About this Class

The Aggregator class runs first-level GLMs on a given BIDS project. 
The user supplies the BIDS root and a functional task of interest 
and the Aggregator does the rest of the heavy lifting.

Ian Richard Ferguson | Stanford Unversity
"""

# ---- Imports
from build_info.build_task_info import build_task_info
from subject.subject import Subject
import nibabel as nib
import bids, os, json
from tqdm import tqdm

# ---- Object definition
class Aggregator:

      # ---- Class functions
      def __init__(self, bids_root, task, template_space='MNI152NLin2009'):
            """
            Constructor method

            Parameters
                  bids_root: str | Relative path to the top of your BIDS project (e.g., './bids')
                  task: str | Functional task from your BIDS project (e.g., 'stopsignal')
                  template_space: str | Template space for your preprocessed data (default='MNI152NLin2009')
            """
            
            self.bids_root = bids_root
            self.task = task
            self.template_space = template_space

            # List of subject ID's in your BIDS project
            self.subjects = self._derive_subjects()


      # ---- Helper functions
      def _derive_subjects(self):
            """
            Returns
                  List of subjects from your BIDS project
            """

            # Instantiate BIDSLayout object with your project root as the argument
            bids_ = bids.BIDSLayout(self.bids_root)

            # Return list of subject ID's
            return bids_.get_subjects()


      def _check_task_file(self):
            """
            Returns
                  Boolean | if True, the requisite task information file exists
            """

            return os.path.exists('./task_information.json')


      def run_all_models(self, smoothing=8., verbose=False, conditions=True, contrasts=True,
                         plot_brains=True, user_design_matrices=None, non_steady_state=False,
                         include_modulators=False, auto_block_regressors=False, motion_outliers=True,
                         drop_fixation=True):
            """
            Loops through all subjects in BIDS project and fits first-level GLMs using
            Subject object and run_first_level_glm function

            Parameters
                  smoothing: float | Smoothing kernel to apply to models (default = 8.)
                  verbose: Boolean | if True, subject-wise model information will print
                  conditions: Boolean | if True, unique trial type values are mapped
                  contrasts: Boolean | if True, default or user-defined contrasts are mapped
                  plot_brains: Boolean | if True, contrast-wise plots and summary files are saved locally
                  user_design_matrices: list | You have the option to supply your own design matrices if you like (list of DataFrames)
                  non_steady_state: Boolean | if True, non-steady-state regressors are automatically included as an aggregate regressor
                  include_modulators: Boolean | if True, user-defined modulators are included in design matrices
                  auto_block_regressors: Boolean | if True, `block_type` and `trial_type` regressors are implicitly merged
                  motion_outliers: Boolean | if True, motion outlier regressors are automatically included as an aggregate regressor
                  drop_fixation: Boolean | if True, explicit fixation trials are dropped from the design
            """

            # Build task information file if it doesn't already exist
            if not self._check_task_file():
                  build_task_info(self.bids_root)
                  print('BE AWARE: task_information.json created ... your models will run with default parameters')

            print(f'\n** Fitting first-level models for {len(self.subjects)} subjects **\n')

            # Loop through subject ID's
            for sub in tqdm(self.subjects):
                  try:
                        # Instantiate each subject as a Subject instance
                        sub_instant = Subject(sub, task=self.task, 
                                          bids_root=self.bids_root,
                                          suppress=True,
                                          template_space=self.template_space)

                  except Exception as e:
                        print(f'Error instantiating sub-{sub} ... {e}')
                        continue

                  try:
                        # Run first-level GLM with your supplied arguments
                        sub_instant.run_first_level_glm(smoothing=smoothing, verbose=verbose,
                                                      conditions=conditions, contrasts=contrasts,
                                                      plot_brains=plot_brains, 
                                                      user_design_matrices=user_design_matrices,
                                                      non_steady_state=non_steady_state,
                                                      include_modulators=include_modulators,
                                                      auto_block_regressors=auto_block_regressors,
                                                      motion_outliers=motion_outliers,
                                                      drop_fixation=drop_fixation)

                  except Exception as e:
                        print(f'Error modeling sub-{sub} ... {e}')
                        continue


#!/bin/python3

"""
About this Class

BIDSPointer is the foundation of the subject-level
analysis pipeline. The subsequent Subject object
inherits this class.

Ian Richard Ferguson | Stanford University
"""

"""
Usage Notes

* The TEMPLATE SPACE attribute is hard-coded ... override this with set_template_space()
* TR is hard-coded at 1. ... override this with set_tr()
* DUMMY SCANS are hard-coded at 2 ... override this with set_dummy_scans()
"""

# ---- Imports
from build_info.build_task_info import build_task_info
import os, json, pathlib
import pandas as pd
from bids import BIDSLayout


# ---- Object definition
class BIDSPointer:
      def __init__(self, sub_id, task, bids_root='./bids', suppress=False, template_space="MNI152NLin6",
                  dummy_scans=0, repetition_time=1.):
            """
            BIDSPointer constructor
            
            Parameters
                  sub_id: str or int | Subject ID corresponding to label in BIDS project
                  task: str | Task name corresponding to label in BIDS project
                  bids_root: str | Relative path to top of BIDS project directory tree
                  suppress: Boolean | if True, dictionary of attributes printed at init
                  template_space: str | Anatomical space derived from fmriprep (defaults to MNI152NLin6)
                  dummy_scans: int | Number of non-steady state volumes preceding functional runs
                  repetition_time: int | TR value derived from fMRI scanner
            """

            self.sub_id = sub_id                                                    # Unique subject ID from BIDS project
            self.task = task                                                        # Task name from BIDS project
            self.template_space = template_space                                    # Preprocessed template space
            self.bids_root = bids_root


            # === Validate BIDS input ===
            bids = BIDSLayout(self.bids_root)                                            

            if self.sub_id not in bids.get_subjects():
                  raise OSError(f'{self.sub_id} not found in BIDS project ... valid: {bids.get_subjects()}')

            if self.task not in bids.get_tasks():
                  raise OSError(f'{self.task} not found in BIDS project ... valid: {bids.get_tasks()}')


            # === BIDS Paths ===  
            self.raw_bold, self.events = self._split_raw_data()                     # Raw (unrprocessed NifTi and Events files)
            self.preprocessed_bold, self.confounds = self._split_derived_data()     # Preprocessed NifTi and confound regressor files
            self.first_level_output = self._output_tree()                           # Relative path for analysis output
            self.functional_runs = self._derive_functional_runs()                   # Length of functional runs for this task
            self.bids_container = self._build_container()                           # Container to organize all NifTi / events / confounds


            # === Object attributes ===
            task_file = self._validate_task_file()                                  # Build task_information.json file if it doesn't exist                             

            self.t_r = repetition_time                                              # User-defined repetition time
            self.dummy_scans = dummy_scans                                          # User-defined dummy scans

            self.conditions = []                                                    # Conditions list (empty @ __init__)
            self.condition_variable = task_file['condition_identifier']             # 
            self.confound_regressors = task_file['confound_regressors']             # List of confound regressors to include
            self.contrasts = task_file['design_contrasts']                          # Dictionary of contrasts to compute

            # Print subject information at __init__
            if not suppress:
                  print(str(self))


      def __str__(self):
            container = {'Subject ID': self.sub_id,
                        'Task': self.task,
                        "# of Functional Runs": self.functional_runs,
                        "Output Directory": self.first_level_output,
                        "Defined Contrasts": self.contrasts,
                        "Confound Regressors": self.confound_regressors}

            return json.dumps(container, indent=4)


      # ---- Setter methods
      def set_tr(self, incoming):
            self.t_r = incoming


      def set_dummy_scans(self, incoming):
            self.dummy_scans = incoming


      def set_conditions(self, incoming):
            if not type(incoming) == list:
                  raise TypeError('Provide this function with a list of condition trial types from your events file')
            self.conditions = incoming


      def set_template_space(self, incoming):
            self.template_space = incoming


      def set_confound_regressors(self, incoming):
            if not type(incoming) == list:
                  raise TypeError('Provide this function with a list of regressors derived from fmriprep')
            self.confound_regressors = incoming


      def set_design_contrasts(self, incoming):
            if not type(incoming) == dict:
                  raise TypeError('Design contrasts need to be a dictionary in {"labeled_name": "column1 - column2" format')

            self.contrasts = incoming


      # ---- Ecosystem helpers
      def _validate_task_file(self):
            """
            Builds task_information.json if it doesn't already exist

            Returns
                  Task-specific parameters from JSON file
            """

            # Relative path to task_information file
            target = os.path.join('./task_information.json')

            # Build task file if it doesn't exist
            if not os.path.exists(target):
                  build_task_info(self.bids_root)

            # Open task file and return as dictionary
            with open(target) as incoming:
                  return json.load(incoming)[self.task]



      def _output_tree(self):
            """
            Builds out subject's directory hierarchy for first-level modeling

            Returns
                  Relative path to first-level output
            """

            # Base output directory
            primary = os.path.join(self.bids_root, 
                                  'derivatives/first-level-output/',
                                  f'sub-{self.sub_id}/task-{self.task}')

            # Build out subdirectories
            for secondary in ['models', 'plots']:
                  for tertiary in ['condition-maps', 'contrast-maps']:
                        temp = os.path.join(primary, secondary, tertiary)

                        if not os.path.exists(temp):
                              pathlib.Path(temp).mkdir(parents=True, exist_ok=True)

            return primary


      # ---- Isolate data paths
      def _split_raw_data(self):
            """
            Isolates relative paths to unprocessed BOLD and EVENTS files

            Returns
                  Two lists (path to BOLD files, path to Events files)
            """

            # Isolate path to subject data in BIDS project (not preprocessed)
            iso_path = [path for path in os.listdir(self.bids_root) if self.sub_id in path][0]
            
            # Path to functional subdirectory
            rel_path = os.path.join(self.bids_root, iso_path, 'func')

            # All files in subject's functional subdirectory
            raw_data = [os.path.join(rel_path, x) for x in os.listdir(rel_path) if self.task in x]
            
            
            # All NifTi files for the current task
            bold = [x for x in raw_data if '.nii.gz' in x]
            
            # All events files for the current task
            events = [x for x in raw_data if '.tsv' in x]

            return bold, events



      def _split_derived_data(self):
            """
            Isolates relative paths to preprocessed BOLD and CONFOUNDS files

            Returns
                  Two lists (path to BOLD files, path to Confounds files)
            """

            # Path to fmriprep derivatvies
            deriv_path = os.path.join(self.bids_root, 'derivatives/fmriprep')
            
            # Isolate path to subject's preprocessed data (excluding the useful HTML files created by fmriprep!)
            iso_path = [path for path in os.listdir(deriv_path) if self.sub_id in path if ".html" not in path][0]
            
            # Path to functional subdirectory in preprocessed directory
            rel_path = os.path.join(deriv_path, iso_path, 'func')

            
            # Preprocessed NifTi's must match template space and given naming conventions
            bold = [os.path.join(rel_path, x) for x in os.listdir(rel_path) if self.task in x 
                    if self.template_space in x 
                    if '.nii.gz' in x
                    if 'preproc' in x]

            # Confounds derived for each functional run
            confounds = [os.path.join(rel_path, x) for x in os.listdir(rel_path) if self.task in x if '.tsv' in x]

            return bold, confounds



      def _derive_functional_runs(self):
            return len(self.raw_bold)


      # ---- Utility helpers
      def _init_container(self):
            """
            Initializes container to store run-specific bold / event / confound info

            Returns
                  Empty BIDS Container, this is filled in the next step
            """
            
            # Empty dictionary to append into
            container = {}

            # E.g., 0, 1, 2
            for run in range(self.functional_runs):

                  # 0 => run-1
                  key = f'run-{run+1}'

                  container[key] = {}

                  for filetype in ['func', 'event', 'confound']:
                        container[key][filetype] = ''

            # These are empty lists at __init__
            container['all_func'] = []
            container['all_events'] = []
            container['all_confounds'] = []

            return container



      def _build_container(self):
            """
            Populates BIDS Container object with information

            Returns
                  BIDS Container ready for analysis
            """

            # Initialize container
            container = self._init_container()

            # Build runwise and aggregated lists 
            for run in range(self.functional_runs):
                  
                  key = f'run-{run+1}'

                  # Subject has multiple functional runs for the current task
                  if len(self.preprocessed_bold) > 1:
                        current_bold = [x for x in self.preprocessed_bold if key in x][0]
                        current_event = [x for x in self.events if key in x][0]
                        current_confound = [x for x in self.confounds if key in x][0]

                  # Subject has a single functional run for the current task
                  else:
                        try:
                              current_bold = [x for x in self.preprocessed_bold][0]

                        # Raise error if no functional runs are found - we assume this to be erroneous
                        except:
                              raise ValueError(f'No functional runs identified for task-{self.task} + template-space-{self.template_space}')
                        
                        
                        #
                        current_event = [x for x in self.events][0]
                        current_confound = [x for x in self.confounds][0]

                  """
                  Philosophy of this step ...

                  * We'll have run-wise keys (e.g., run-1, run-2, etc.) that will store relative paths to preprocessed BOLD maps, events, and confounds
                  * We'll additionally have format-wise keys (e.g., all_functional, all_events, all_confounds)
                  """

                  # Populate run-wise keys
                  container[key]['func'] = current_bold
                  container[key]['event'] = current_event
                  container[key]['confound'] = current_confound

                  # Append to format-wise keys
                  container['all_func'].append(current_bold)
                  container['all_events'].append(current_event)
                  container['all_confounds'].append(current_confound)

            # Save local BIDS container in subject's first-level-output directory
            with open(f'{self.first_level_output}/sub-{self.sub_id}_task-{self.task}_bids-container.json', 'w') as outgoing:
                  json.dump(container, outgoing, indent=5)

            return container



      def load_events(self, run='ALL'):
            """
            Loads events file (or files!) in a Pandas DataFrame

            Parameters
                  run: int or str | Defaults to ALL, or else enter a single run number

            Returns
                  Pandas DataFrame
            """
            
            if run != 'ALL':

                  # Isolate event file from all events if there are multiple runs
                  iso_event = [x for x in self.bids_container['all_events'] if f'run-{run}' in x]
                  
                  if len(iso_event) == 0:
                        # Isolate event file from all events if there is one run
                        iso_event = [x for x in self.bids_container['all_events']]
                        
                  # Read in Event file as Pandas DataFrame      
                  temp_event = pd.read_csv(iso_event[0], sep='\t')

                  # Add run column if it doesn't exist
                  if 'run' not in temp_event.columns:
                        temp_event['run'] = [int(run)] * len(temp_event)

                  return temp_event

            else:
                  # Empty list to append into
                  all_events = []

                  for k in range(self.functional_runs):
                        
                        # Index 0 == run-1
                        run = k+1

                        # Read in event file as Pandas DataFrame
                        temp = self.bids_container[f'run-{run}']['event']
                        temp = pd.read_csv(temp, sep='\t')

                        # Add run column if it doesn't exist
                        if 'run' not in temp.columns:
                              temp['run'] = [run] * len(temp)

                        # Add temporary DataFrame to list
                        all_events.append(temp)

                  # Concatenate temporary frames into one DataFrame
                  agg_events = pd.concat(all_events).reset_index(drop=True)

                  return agg_events



      def load_confounds(self, run='ALL'):
            """
            Loads confounds into a Pandas DataFrame object

            Parameters
                  run: int or str | Defaults to ALL, or else enter a single run number

            Returns
                  Pandas DataFrame
            """

            if run != 'ALL':
                  # Isolate confounds TSV file
                  iso_confound = [x for x in self.bids_container['all_confounds'] if f'run-{run}' in x]

                  if len(iso_confound) == 0:
                        iso_confound = [x for x in self.bids_container['all_confounds']]

                  # Read file as Pandas DataFrame
                  temp_confound = pd.read_csv(iso_confound[0], sep='\t')

                  # Add run column if it doesn't exist
                  if run not in temp_confound.columns:
                        temp_confound['run'] = [int(run)] * len(temp_confound)

                  return temp_confound

            else:
                  # Empty list to append into
                  all_confounds = []

                  for k in range(self.functional_runs):
                        
                        # Index 0 == run-1
                        run = k+1

                        # Read in confound file as Pandas DataFrame
                        temp = self.bids_container[f'run-{run}']['confound']
                        temp = pd.read_csv(temp, sep='\t')

                        # Add run column if it doesn't exist
                        if 'run' not in temp.columns:
                              temp['run'] = [run] * len(temp)

                        # Add temporary DataFrame to list
                        all_confounds.append(temp)

                  # Concatenate temporary frames into one DataFrame
                  agg_confounds = pd.concat(all_confounds).reset_index(drop=True)

                  return agg_confounds


#!/bin/python3

"""
About this Class
"""

import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)
warnings.simplefilter(action='ignore', category=UserWarning)

from subject.subject import Subject
from group_level.group import GroupLevel
from aggregator.aggregator import Aggregator


#!/bin/python3

"""
About this Class

GroupLevel aggregates first-level contrast maps derived from the Subject class.

Ian Richard Ferguson | Stanford University
"""

# ---- Imports
import math, os, json, random, pathlib, glob
import pandas as pd
import numpy as np
from tqdm import tqdm
from nilearn.glm import second_level, threshold_stats_img
import nilearn.plotting as nip
import matplotlib.pyplot as plt
from nilearn import image
from bids.layout import BIDSLayout


# ---- Object definition
class GroupLevel:
      def __init__(self, task, bids_root='./bids'):
            """
            Parameters
                  task: str | Corresponds to functional task in your BIDS project
                  bids_root: str | Relative path to BIDS project
            """

            # === BIDS paths ===
            self.bids_root = bids_root
            self._layout = BIDSLayout(self.bids_root)
            self.subjects = self._layout.get_subjects()
            self._all_tasks = self._layout.get_tasks()

            # === Object attributes ===
            self.task = task
            self.output_path = self._build_output_directory
            self.all_contrasts = self._all_available_contrasts()
            self.task_file = self._taskfile_validator()



      def _taskfile_validator(self):
            """
            Confirms existence of task_information.json. This file should have been created
            in the Subject object!

            Returns
                  Task-specific parameters from JSON file
            """

            if not os.path.exists('./task_information.json'):
                  raise OSError(f'Task information JSON file not found ... your directory: {os.getcwd()}')

            with open('./task_information.json') as incoming:
                  return json.load(incoming)[self.task]



      def _build_output_directory(self):
            """
            Builds out group-level directory hierarchy for second-level modeling

            Returns
                  Relative path to second-level output
            """

            # Base output directory
            target = os.path.join(self.bids_root, f'derivatives/second-level-output/task-{self.task}')

            # Create directory if it doesn't exist
            if not os.path.exists(target):
                  pathlib.Path(target).mkdir(exist_ok=True, parents=True)

            # Create corrected / uncorrected subdirs for models and plots
            for secondary in ['models', 'plots']:
                  for tertiary in ['corrected', 'uncorrected']:
                        temp = os.path.join(target, secondary, tertiary)

                        if not os.path.exists(temp):
                              pathlib.Path(temp).mkdir(exist_ok=True, parents=True)

            return target



      def _iso_BIDSsubjects(self):
            """
            Aggregates list of subjects that have undergone first-level modeling

            Returns
                  List of first-level modeled subjects
            """

            # Empty list to append into
            task_subjects = []
            
            # Output directory from first-level modeling
            first_level = os.path.join(self.bids_root, 'derivatives/first-level-output')

            # Loop through subjects ... if they have output for the current task add them to the list
            for sub in self.subjects:
                  temp = os.path.join(first_level, f'sub-{sub}', f'task-{self.task}')

                  if os.path.isdir(temp):
                        task_subjects.append(f'sub-{sub}')

            return task_subjects



      def get_brain_data(self, contrast, smoothing, discard_modulated=True):
            """
            Parameters
                  contrast: str | Contrast name derived from first-level modeling
                  discard_modulated: Boolean | if True, modulated maps not included implicitly  

            Returns
                  List of relative paths to contrast maps
            """

            # Output directory from first-level modeling
            derivatives = os.path.join(self.bids_root, 'derivatives/first-level-output')

            # E.g., 8. => 8mm
            smooth_string = f'{int(smoothing)}mm'
            
            # List of NifTi files that match given contrast
            brain_maps = [x for x in glob.glob(f'{derivatives}/**/*.nii.gz', recursive=True) 
                         if contrast in x if smooth_string in x]

            """
            If True, modulated contrasts are not included unless explicitly specificed

            E.g.,
            If False:   high_trust_x_target, high_trust
            If True:    high_trust
            """

            if discard_modulated:
                  brain_maps = [x for x in brain_maps if f'{contrast}_x_' not in x]

            return brain_maps



      def _all_available_contrasts(self):
            """
            Selects random participant, lists out their modeled conditions and contrasts,
            and aggregates in a dictionary

            TODO: Select second target and compare, to make sure no maps are missing

            Returns
                  Dictionary of conditions and contrasts
            """

            def iso_information(x):
                  return x.split('/')[-1].split('_')[1]

            # Base dictionary
            output = {'conditions': [], 'contrasts': []}

            # TODO: Improve this appoach to be less subjective
            random_target = random.choice(self.subjects)

            # First-level models from target subject
            target = os.path.join(self.bids_root, 'derivatives/first-level-output',
                                 f'sub-{random_target}', f'task-{self.task}/models')

            # TODO: Subset the contrast names out (currently full file names)
            # Add conditions and contrasts to dictionary
            conditions = [x for x in glob.glob(f'{target}/condition-maps/*.nii.gz', recursive=True)]
            output['conditions'] = [iso_information(x) for x in conditions]

            contrasts = [x for x in glob.glob(f'{target}/contrast-maps/*.nii.gz', recursive=True)]
            output['contrasts'] = [iso_information(x) for x in contrasts]

            return output



      # ---- Utility functions
      def plot_brain_mosaic(self, contrast, smoothing=8., save_local=False):
            """
            Parameters
                  contrast: str | Contrast name from first-level model
                  smoothing: float | Smoothing kernel from first-level model (this is a naming convention parameter)
                  save_local: Boolean | if True, saves to second-level-output directory
            """

            # List of relative paths to relevant NifTi files
            brain_maps = self.get_brain_data(contrast=contrast, smoothing=smoothing)

            # E.g., 8. => '8mm
            check_smooth = f'{int(smoothing)}mm'

            # Subset of NifTi files that match given smoothing kernel
            brain_maps = [x for x in brain_maps if check_smooth in x]


            # === Plot brains ===
            # Subplots rows
            rows = int(math.ceil(len(brain_maps) / 5))

            # Instantiate figure cavnas
            figure, axes = plt.subplots(nrows=rows, ncols=5, figsize=(20,20))

            if rows > 1:
                  for x in range(rows):
                        for y in range(5):
                              axes[x,y].axis('off')

            else:
                  for y in range(5):
                        axes[y].axis('off')

            figure.suptitle(f'{contrast}_{int(smoothing)}mm', fontweight='bold')

            for ix, brain in enumerate(brain_maps):

                  # Parse out subject ID
                  sub_id = brain.split('/')[-1].split('_')[0].split('-')[1]

                  # Slightly different logic depending on the shape of the output file (see axes)
                  if rows > 1:
                        k = nip.plot_glass_brain(brain, threshold=3.2, display_mode='z',
                                                plot_abs=False, colorbar=False, title=sub_id,
                                                axes=axes[int(ix/5), int(ix % 5)])

                  else:
                        k = nip.plot_glass_brain(brain, threshold=3.2, display_mode='z',
                                                plot_abs=False, colorbar=False, title=sub_id,
                                                axes=axes[int(ix % 5)])

            # Save plot locally if True
            if save_local:
                  filename = os.path.join(self.output_path, 'plots', f'brain_mosaic_{contrast}_{int(smoothing)}mm.jpg')
                  plt.savefig(filename)

            plt.show()



      # ---- Modeling functions
      def easy_loader(self, contrast, smoothing=8.):
            """
            Parameters
                  contrast: str | Contrast map derived from first-level model
                  smoothing: float | Smoothing kernel applied to first-level maps

            Returns
                  List of relative paths to all contrast-specific contrast maps
            """

            # Pattern to match in glob function
            f_path = f'{self.bids_root}/derivatives/first-level-output/**/*.nii.gz'

            # E.g., 8. => 8mm ... for naming convention purposes
            f_smooth = f'{int(smoothing)}mm'

            # Returns list of brain maps
            return [x for x in glob.glob(f_path) if contrast in x if f_smooth in x]



      def make_design_matrix(self, direction=1):
            """
            Parameters
                  direction: int | 1 or -1 (determines direction of linear contrast)

            Returns
                  Pandas DataFrame object
            """

            if direction not in [1, -1]:
                  raise ValueError(f'Direction must be 1 or -1 ... your input: {direction}')

            # List of subject id's
            subject_labels = self.subjects
            
            # [1] or [-1] * length of subject set
            design_matrix = pd.DataFrame({'subject_label': subject_labels,
                                           'intercept': [direction] * len(subject_labels)})

            return second_level.make_second_level_design_matrix(subject_labels, design_matrix)



      def uncorrected_model(self, contrast, sub_smoothing=8., group_smoothing=None, direction=1,
                           return_map=True, save_output=False):
            """
            Parameters
                  contrast: str | First-level contrast
                  sub_smoothing: float | Smoothing kernel used in first-level models
                  group_smoothing: float | Smoothing kernel to be applied to second-level model
                  direction: int | 1 or -1, determines direction of linear contrast

            Returns
                  nibabel.Image object (if return_map)
            """

            if direction not in [1, -1]:
                  raise ValueError(f'Direction must be 1 or -1 ... your input: {direction}')

            # List of brain maps matching contrast and first-level smoothing kernel
            brain_data = self.get_brain_data(contrast=contrast, smoothing=sub_smoothing)

            # Make second-level design matrix
            #design_matrix = self.make_design_matrix(direction=direction)
            design_matrix = pd.DataFrame([direction] * len(brain_data))

            print(f'\n ** {len(brain_data)} subjects in {contrast.upper()} group-level model **\n')

            # Instantiate SecondLevelModel object
            glm = second_level.SecondLevelModel(smoothing_fwhm=group_smoothing)

            # Fit brain data and 
            model = glm.fit(brain_data, design_matrix=design_matrix)

            # Run linear contrast on NifTi Image
            contrasted_model = model.compute_contrast(output_type='z_score')

            # Save local NifTi if True
            if save_output:

                  if group_smoothing is not None:
                        #
                        smooth_string = f'{int(group_smoothing)}mm'

                        #
                        output_path = os.path.join(self.output_path, 
                                                  'models', 
                                                  f'second_level_contrast-{contrast}_smoothing-{smooth_string}.nii.gz')

                        #
                        contrasted_model.to_filename(output_path)

                  else:
                        #
                        output_path = os.path.join(self.output_path,
                                                  'models',
                                                  f'second_level_contrast-{contrast}_unsmoothed.nii.gz')

                        #
                        contrasted_model.to_filename(output_path)

            #
            if return_map:
                  return contrasted_model



      def corrected_model(self, contrast, existing_model=None, height_control='fdr', alpha=.05,
                         plot_style='ortho', cluster_threshold=None, sub_smoothing=8.,
                         group_smoothing=None, direction=1, return_map=True, save_output=False):
            """
            Parameters
                  contrast: str | Valid contrast from first level model 
                  existing_model: SecondLevelModel object | This allows you to feed in uncorrected stats image
                  height_control: str | Must be in ['fpr', 'fdr', 'bonferroni']
                  alpha: float | P-value cutoff parameter for thresholding
                  plot_style: str | Must be in ['ortho', 'glass']
                  cluster_threshold: int | Only required for FPR height control models
                  sub_smoothing: float |
                  group_smoothing: float |
                  direction: int | Must be in [1, -1]
                  reutrn_map: Boolean | if True, stats image is returned
                  save_output: Boolean | if True, plot is saved to output directory

            Returns
                  nibabel.Image object (if return_map)
            """

            # Catch any erroneous user inputs
            if height_control not in ['fdr', 'fpr', 'bonferroni']:
                  raise ValueError(f'Invalid height control parameter {height_control} supplied...')

            if plot_style not in ['ortho', 'glass']:
                  raise ValueError(f'Invalid plot_style parameter {plot_style} supplied...')

            if direction not in [1, -1]:
                  raise ValueError(f'Invalid contrast direction {direction} supplied...')


            # Fit model if none supplied
            if existing_model == None:

                  # List of NifTi files
                  brain_data = self.easy_loader(contrast=contrast, smoothing=sub_smoothing)
                  
                  # Build design matrix
                  design_matrix = self.make_design_matrix(direction=direction)
                  
                  # Instantiate SecondLevelModel
                  glm = second_level.SecondLevelModel(smoothing_fwhm=group_smoothing)
                  
                  # Fit to NifTi data
                  model = glm.fit(brain_data, design_matrix=design_matrix)

            else:
                  model = existing_model


            # === Modeling parameters ===
            if height_control != 'fpr':
                  t_map, t_thresh = threshold_stats_img(model, alpha=alpha, 
                                                       height_control=height_control)
            
            else:
                  t_map, t_thresh = threshold_stats_img(model, alpha=alpha,
                                                      height_control=height_control,
                                                      cluster_threshold=cluster_threshold)


            title = f'{contrast} @ {alpha}'

            if save_output:
                  
                  # Output name for plot
                  plot_filename = os.path.join(self.output_path, 'plots', f'{contrast}_corrected.jpg')
                  
                  # In development
                  # model_filename = os.path.join(self.output_path, 'models', f'{contrast}_corrected.nii.gz')

                  if plot_style == 'ortho':
                        nip.plot_stat_map(t_map, threshold=t_thresh, title=title,
                                          display_mode='ortho', output_file=plot_filename)

                  elif plot_style == 'glass':
                        nip.plot_glass_brain(t_map, threshold=t_thresh, display_mode='lyrz',
                                             plot_abs=False, colorbar=False, title=title,
                                             output_file=plot_filename)


#!/bin/python3

"""
About this Class

Subject inherits all functions and attributes from the BIDSPointer class defined in this package. 
Building on this framework, Subject is optimized to run basic first-level GLMs out of the box, 
and users have the option to define their own design matrices as well.

Ian Richard Ferguson | Stanford University
"""

# ---- Imports
from .bids_pointer import BIDSPointer
import os, json, pathlib
from time import sleep
import pandas as pd
import numpy as np
from tqdm import tqdm
import nilearn.plotting as nip
from nilearn.glm import first_level
from nilearn.reporting import make_glm_report

# ---- Object definition
class Subject(BIDSPointer):

      # ---- Class functions
      def __init__(self, sub_id, task, bids_root, suppress=False, template_space="MNI152NLin2009",
                  repetition_time=1., dummy_scans=0):

            # Inherits constructor from BIDSPointer
            BIDSPointer.__init__(self, sub_id=sub_id, task=task, bids_root=bids_root, suppress=suppress,
                                template_space=template_space, repetition_time=repetition_time,
                                dummy_scans=dummy_scans)

            # Boolean - are modulators provided or not?
            self.has_modulators = self._has_modulators()

            if self.has_modulators:

                  # Read in modulators from task_information.json file
                  self.modulators = self._parse_modulators()
                  
                  # Get mean values for each modulator
                  self._center_modulators()


      # ---- Modeling helpers
      def _parse_modulators(self):
            """
            Reads in modulators from task_information.json

            Returns
                  List of user provided modulators
            """

            with open('./task_information.json') as incoming:
                  data = json.load(incoming)[self.task]

            return list(data['modulators'])


      def _has_modulators(self):
            """
            Returns
                  Boolean | if True, modulators were provided
            """

            with open('./task_information.json') as incoming:
                  data = json.load(incoming)[self.task]

            return len(data['modulators']) > 1


      def _center_modulators(self):
            """
            Calculates average values from modulators, to use for centering in design stage
            """

            # Read in all events
            all_events = self.load_events()

            # Modulators key = empty dictionary to append into
            self.bids_container['modulators'] = {}

            # Loop through user-provided modulators
            for mod in self.modulators:
                  
                  """
                  NOTE: We assume that all modulators are contained in your events file!

                  If your modulators are not in the events file, they'll be removed
                  from the Subject object
                  """
                  
                  if mod not in all_events.columns:
                        print(f'\n** ERROR: {mod.upper()} not in Events File\n')
                        self.modulators.remove(mod)
                        continue

                  # Mean value from modulator column
                  avg = all_events[mod].mean()

                  # Assign to bids_container object
                  self.bids_container['modulators'][mod] = avg



      # ---- Modeling functions
      def generate_design_matrix(self, run, non_steady_state=False, include_modulators=False, 
                                 auto_block_regressors=False, motion_outliers=True, drop_fixation=True):
            """
            Builds a first level design matrix

            Parameters
                  run: int | Functional run number, lines up with events file
                  non_steady_state: boolean | if True, all non steady state regressors are aggregated as one regressor
                  include_modulators: boolean | if True, Parametric modulators are calculated and included in design matrix
                  auto_block_regressors: boolean | if True, `block_type` and `trial_type` columns are implicitly merged
                  motion_outliers: boolean | if True, all motion outliers are included in design matrix
                  drop_fixation: boolean | if True, fixation trials are dropped from event files

            Returns
                  Pandas DataFrame object
            """

            # -- Helpers
            def aggregate_non_steady_state(length, dummy_value):
                  """
                  Creates a binary vector to serve as non_steady_state regressor

                  Parameters
                        length: int | Number of scans in the present run
                        dummy_value: int | Number of non-steady state regressors 

                  Returns
                        A binary vector of length `length`
                  """

                  # Number of scans - Number of dummy scans
                  real_length = length - int(dummy_value)

                  # Binary vector
                  return [1] * dummy_value + [0] * real_length


            def block_regressor(DF):
                  """
                  Aggregates block and trial regressors (e.g., high_trust_perspective)

                  Parameters
                        DF: Pandas DataFrame | Both block and trial type must be present for this function to work!

                  Returns
                        Formatted string per row
                  """

                  # Block regressor
                  block = DF['block_type']
                  
                  # Trial type regressor
                  trial = DF['trial_type']

                  # Return formatted string
                  return f'{block}_{trial}'


            def create_temp_dataframe(events, mod_name, frame_times):
                  """
                  Creates temporary DataFrame to merge into events DF (optimized to handle modulators)

                  Parameters
                        events: Pandas DataFrame |
                        mod_name: str |
                        frame_times: int |

                  Returns
                        Reduced Pandas DataFrame
                  """

                  # Nilearn implicitly requires a modulation column - we'll rename the column of interest here
                  temp = events.rename(columns={mod_name: 'modulation'})

                  # Calculate distance from the mean for the current modulation
                  temp['modulation'] = temp['modulation'] - self.bids_container['modulators'][mod_name]

                  # Rename trial_type => trial_type_x_{modulation}
                  temp['trial_type'] = temp['trial_type'].apply(lambda x: f'{x}_x_{mod_name}')

                  # These will be the only columns we output
                  output_columns = list(temp['trial_type'].unique())

                  # Translate DataFrame to Design Matrix
                  dm = first_level.make_first_level_design_matrix(frame_times,
                                                                  temp,
                                                                  hrf_model='spm')

                  # Return isolated trial x modulation columns AND index
                  return dm.loc[:, output_columns].reset_index()


            def reorder_design_columns(DM):
                  """
                  Moves Nilearn modeling regressors to the end of the design matrix

                  Parameters
                        DM: Pandas DataFrame |

                  Returns:
                        Pandas DataFrame
                  """

                  # Drop index column if it exists
                  try:
                        DM.drop(columns=['index'], inplace=True)
                  except:
                        DM = DM

                  # Nilearn modeling regressors (drift and constant)
                  tail = [x for x in DM.columns if 'drift' in x] + ['constant']
                  
                  # All other column names
                  head = [x for x in DM.columns if x not in tail]

                  # Head first, then tails
                  clean = head + tail

                  # Return reordered Design Matrix
                  return DM.loc[:, clean]


            # === Model foundations ===
            iso_container = self.bids_container[f'run-{run}']                                   # BIDS container for the current run
            events = pd.read_csv(iso_container['event'], sep='\t')                              # Load events file
            confounds = pd.read_csv(iso_container['confound'], sep='\t')                        # Load fmriprep regressors
            voi = ['onset', 'duration', 'trial_type']                                           # Starting variables of interest

            if auto_block_regressors:
                  if 'block_type' in events.columns:
                        voi += ['block_type']                                                   # Add block type to VOI if True

            if include_modulators:
                  voi += self.modulators                                                        # Add modulators to VOI if True

            events = events.loc[:, voi]                                                         # Reduce 
            

            # === Model parameters ===
            n_scans = len(confounds)                                                            # Number of scans in the current run
            t_r = self.t_r                                                                      # Repitition time
            frame_times = np.arange(n_scans) * t_r                                              # Volume acquisition sequence


            # Drop explicit fixation trials if True
            if drop_fixation:
                  events = events[events['trial_type'] != 'fixation'].reset_index(drop=True)

            # Aggregate block_type and trial_type regressors if True
            if auto_block_regressors:
                  events['trial_type'] = events.apply(block_regressor, axis=1)
                  events.drop(columns=['block_type'], inplace=True)

            # Create mod-wise Design Matrices if True
            if include_modulators:
                  mod_matrices = []

                  for novel_modulator in self.modulators:
                        mod_matrices.append(create_temp_dataframe(events, novel_modulator, frame_times))


            # Set object conditions to match trial types
            clean_conditions = list(events['trial_type'].dropna().unique())
            self.set_conditions(clean_conditions)

            # Create baseline Design Matrix with defined parameters
            events = first_level.make_first_level_design_matrix(frame_times, events, hrf_model='spm').reset_index()

            # Merge modulated DataFrames into Design Matrix if True
            if include_modulators:
                  for matrix in mod_matrices:
                        events = events.merge(matrix, on='index', how='left')

            # We assume there are no dummy scans unless otherwise specified
            try:
                  dummy_value = iso_container['dummy']
            except:
                  dummy_value = 0

            # Base list of confound regressors
            confound_vars = self.confound_regressors.copy()

            # Add non_steady_state regressor to confounds if True
            if non_steady_state:
                  confounds['non_steady_state'] = aggregate_non_steady_state(length=n_scans, dummy_value=dummy_value)
                  confound_vars += ['non_steady_state']

            # Pull in motion outliers to confounds if True
            if motion_outliers:
                  confound_vars += [x for x in list(confounds.columns) if 'motion_outlier' in x]

            # Isolate confound regressor
            confounds = confounds.loc[:, confound_vars].reset_index()

            # Merge events and confounds into one DataFrame
            events = events.merge(confounds, on='index', how='left')

            # We'll ONLY mean impute missing whole-brain motion regressors (FD and DVARS)            
            for var in events.columns:

                  if 'dvars' in var.lower():
                        try:
                              events[var].fillna(np.mean(events[var]), inplace=True)
                        except:
                              continue
                  elif 'framewise' in var.lower():
                        try:
                             events[var].fillna(np.mean(events[var]), inplace=True)
                        except:
                             continue
                        

            # Rearrange Design Matrix columns
            events = reorder_design_columns(events)

            return events



      def first_level_design(self, non_steady_state=False, include_modulators=False, auto_block_regressors=False,
                            motion_outliers=True, drop_fixation=True, verbose=True):
            """
            Compiles one design matrix per functional run

            Parameters
                  non_steady_state: boolean | if True, all non steady state regressors are aggregated as one regressor
                  include_modulators: boolean | if True, Parametric modulators are calculated and included in design matrix
                  auto_block_regressors: boolean | if True, `block_type` and `trial_type` columns are implicitly merged
                  motion_outliers: boolean | if True, all motion outliers are included in design matrix
                  drop_fixation: boolean | if True, fixation trials are dropped from event files

            Returns
                  List of Pandas DataFrame objects (one per functional run)
            """

            model_specs = f"""
            \n\nRunning first-level designs for {self.task.upper()} with the following parameters:\n\n

            Non-steady state regressors:\t{non_steady_state}\n
            Modulators:\t\t\t\t{include_modulators}\n
            Auto-block regressors:\t\t{auto_block_regressors}\n
            Motion outliers:\t\t\t{motion_outliers}\n
            Fixation trials:\t\t\t{drop_fixation}
            \n\n
            """
            
            if verbose:
                  print(model_specs)
            sleep(1)

            # Empty list to append into
            matrices = []

            # Loop through functional runs
            for k in range(self.functional_runs):
                  
                  # k = 0 => run-1
                  run = k+1

                  # Generate design matrix given the input parameters
                  matrix = self.generate_design_matrix(run, non_steady_state, include_modulators,
                                                      auto_block_regressors, motion_outliers, drop_fixation)

                  # Output filename
                  filename = f'sub-{self.sub_id}_task-{self.task}_run-{run}_design-matrix.jpg'

                  # Aggregate output file path
                  output_path = os.path.join(self.first_level_output, 'plots', filename)

                  # Save design matrix plot to output directory
                  nip.plot_design_matrix(matrix, output_file=output_path)

                  # Add Design Matrix to matrices list
                  matrices.append(matrix)

            return matrices


      def _default_contrasts(self):
            """
            Creates trial-wise contrast map to feed into first-level GLM

            Returns
                  Dictionary (keys and values == contrast to compute)
            """

            # Empty dictionary to append into
            contrasts = {}

            # Match trial types as 1-1 contrasts
            for outer in self.conditions:
                  for inner in self.conditions:
                        if outer != inner:

                              # Avoid duplicates - these can be modeled using directionality
                              if (f'{inner} - {outer}') not in list(contrasts.keys()):
                                    k = f'{outer} - {inner}'
                                    contrasts[k] = k

            return contrasts



      def _run_contrast(self, glm, contrast, title, output_type, smoothing, plot_brains=False):
            """
            Linear contrast based on first-level GLM. NifTi output is always saved, stat maps are conditional

            Parameters
                  glm: FirstLevelModel object | Model is fit outside of this function
                  contrast: str | Mathematical contrast to compute (e.g., "high_trust_perspective - self_perspective")
                  title: str | Nominal contrast to feed into title of plot (e.g., "perspective_contrast")
                  output_type: str | Must be in ['condition', 'contrast']
                  smoothing: float | Smoothing kernel from GLM
                  plot_brains: boolean | if True, stat maps and contrast reports are saved locally
            """

            # Remove any white space from contrast
            contrast = str(contrast).replace(' ', '').strip()
            
            # Cast kernel to string
            kernel = str(int(smoothing))

            if output_type == 'condition':
                  model_base = os.path.join(self.first_level_output, 'models', 'condition-maps')
                  plot_base = os.path.join(self.first_level_output, 'plots', 'condition-maps')

            elif output_type == 'contrast':
                  model_base = os.path.join(self.first_level_output, 'models', 'contrast-maps')
                  plot_base = os.path.join(self.first_level_output, 'plots', 'contrast-maps')

            else:
                  raise ValueError(f'Output type {output_type} invalid - must be condition or contrast')

            
            # Output name and path for computed contrast
            nifti_output = f'sub-{self.sub_id}_{output_type}-{title}_smoothing-{kernel}mm_z-map.nii.gz'
            nifti_output = os.path.join(model_base, nifti_output)

            # Run the contrast itself
            z_map = glm.compute_contrast(contrast)

            # Save contrast to output directory
            z_map.to_filename(nifti_output)

            # Plot stat map and reports if True
            if plot_brains:

                  # Formatted output paths
                  stat_output = f'{plot_base}/sub-{self.sub_id}_{output_type}-{title}_smoothing-{kernel}mm_plot-stat-map.png'
                  report_output = f'{plot_base}/sub-{self.sub_id}_{output_type}-{title}_smoothing-{kernel}mm_contrast-summary.html'

                  # Plot stat map
                  nip.plot_stat_map(z_map, threshold=2.3, colorbar=False, draw_cross=False,
                                    display_mode='ortho', title=title,
                                    output_file=stat_output)

                  # Make GLM report
                  make_glm_report(model=glm, contrasts=contrast,
                                plot_type='glass').save_as_html(report_output)



      def run_first_level_glm(self, conditions=True, contrasts=True, smoothing=8., plot_brains=True, user_design_matrices=None,
                              non_steady_state=False, include_modulators=False, auto_block_regressors=False,
                              motion_outliers=True, drop_fixation=True, verbose=True):

            """
            Instantiates and fits a FirstLevelModel GLM object, compiles condition and contrast z-maps

            Parameters
                  conditions: boolean | if True, condition-wise contrast maps are computed
                  contrasts: boolean | if True, contrast-wise contrast maps are computed
                  smoothing: float | Kernel size to apply to contrast maps, default = 8.
                  plot_brains: boolean | if True, local contrast images are saved to the first level output directory
                  user_design_matrices: Defaults to None | if supplied, these replace the Object-generated design matrices
                  non_steady_state: boolean | if True, all non steady state regressors are aggregated as one regressor
                  include_modulators: boolean | if True, Parametric modulators are calculated and included in design matrix
                  auto_block_regressors: boolean | if True, `block_type` and `trial_type` columns are implicitly merged
                  motion_outliers: boolean | if True, all motion outliers are included in design matrix
                  drop_fixation: boolean | if True, fixation trials are dropped from events file
            """

            # === Define DMs and contrasts ===
            # Users have the option of specifying their own design matrices if they like
            if user_design_matrices is not None:
                  matrices = user_design_matrices

            else:
                  matrices = self.first_level_design(non_steady_state=non_steady_state, include_modulators=include_modulators,
                                                   auto_block_regressors=auto_block_regressors, motion_outliers=motion_outliers,
                                                   drop_fixation=drop_fixation, verbose=verbose)
            
            # If user has not provided updated contrast map, the default pairwsie contrasts will run
            if self.contrasts == 'default':
                  contrasts_to_map = self._default_contrasts()

            else:
                  contrasts_to_map = self.contrasts


            # === Data type checks ===
            if isinstance(matrices, list):
                  if not isinstance(matrices[0], pd.DataFrame):
                        raise TypeError('Design matrices should be Pandas DataFrame objects!')

            elif not isinstance(matrices, pd.DataFrame):
                  raise TypeError('Design matrices should be Pandas DataFrame objects!')

            
            # === Build first-level GLM ===
            glm = first_level.FirstLevelModel(t_r=self.t_r, smoothing_fwhm=smoothing, hrf_model='spm', minimize_memory=False)

            if verbose:
                  print('\n=== Fitting GLM ===')

            # Fit to functional runs and design matrices
            model = glm.fit(self.bids_container['all_func'], design_matrices=matrices)

            # === Conditionally run condition and contrast maps ===
            if verbose:
                  disable = False
            else:
                  disable = True


            if conditions:
                  if verbose:
                        print('\n=== Mapping condition z-scores ===\n')
                  
                  for condition in tqdm(self.conditions, disable=disable):
                        self._run_contrast(glm=model, contrast=condition, title=condition, output_type='condition',
                                          smoothing=smoothing, plot_brains=plot_brains)

            if contrasts:
                  if verbose:
                        print('\n=== Mapping contrast z-scores ===\n')

                  for k in tqdm(list(contrasts_to_map.keys()), disable=disable):
                        self._run_contrast(glm=model, contrast=contrasts_to_map[k], title=k,
                                           output_type='contrast', smoothing=smoothing, plot_brains=plot_brains)

            if verbose:
                  print(f'\n\n=== {self.task.upper()} contrasts computed! Subject {self.sub_id} has been mapped ===\n\n') 



#!/bin/python3

"""
About this Script

The `build_task_info()` function creates the relevant `task_information.json` file
that is critical to running first- and second-level models using GLM-Express

Ian Richard Ferguson | Stanford University
"""

import os, json, sys
from bids import BIDSLayout

def derive_tasks(bids_root):
      """
      Feeds user-supplied BIDS root into BIDSLayout object to obtain list of functional tasks

      Parameters
            bids_root: str | Relative path to the top of your BIDS project

      Returns
            List of functional tasks
      """
      
      # Feed bids_root into BIDSLayout object
      # NOTE: This will throw an error if your project is not BIDS-compliant
      bids = BIDSLayout(bids_root)
      
      # Return functional tasks in list
      return bids.get_tasks()


def build_task_info(bids_root):
      """
      This function constructs the `task_information.json` file that is required
      for the Subject and GroupLevel objects to run successfully

      Parameters
            bids_root: str | Relative path to the top of your BIDS project
      """

      # Derive list of tasks in your BIDS project
      functional_tasks = derive_tasks(bids_root)

      # Empty dictionary to append into
      output = {}

      # Loop through tasks and create dictionaries for each
      for task in functional_tasks:

            output[task] = {}                                           # Empty dictionary to append into

            output[task]['block_identifier'] = 'block_type'             # Column in events file corresponding to Block identifier (if applicable) 
            output[task]['condition_identifier'] = 'trial_type'         # Column in events file corresponding to Trial Type identifier
            output[task]['confound_regressors'] = []                    # Regressors to include from fmriprep confounds file
            output[task]['design_contrasts'] = 'default'                # Contrasts of interest to run in first-level design
            output[task]['excludes'] = []                               # List of subject IDs to exclude (for batching / SLURM scheduling)
            output[task]['group_level_regressors'] = []                 # Regressors to include at second-level (NOTE: In development)
            output[task]['modulators'] = []                             # Parametric modulators to include in the first-level analysis (should be columns in events file)
            output[task]['tr'] = 1.                                     # Repetition time for the given first-level task (you can update this in the Subject object too)

      # Save task file locally
      with open(os.path.join('./task_information.json'), 'w') as outgoing:
            json.dump(output, outgoing, indent=6)


if __name__ == "__main__":
      try:
            root = sys.argv[1]
      except:
            raise OSError(f'No command line argument supplied')

      build_task_info(root)



#!/bin/python
from setuptools import setup
import pathlib

HERE = pathlib.Path(__file__).parent

README = (HERE / "glm_express/README.md").read_text()

setup(name='glm_express',
      version='1.0.0',
      description='Automated linear models for functional neuroimaging data',
      long_description=README,
      long_description_content_type="text/markdown",
      url='https://github.com/IanRFerguson/glm-express',
      author='Ian Richard Ferguson',
      author_email='IRF229@nyu.edu',
      license="MIT",
      classifiers=[
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.7"
      ],
      packages=[
            'glm_express',
            'glm_express/aggregator',
            'glm_express/build_info',
            'glm_express/group_level',
            'glm_express/subject'
            ],
      include_package_data=True,
      install_requires=[
            'nilearn', 'pandas', 'tqdm', 'matplotlib', 'numpy', 'pybids'
            ]
      )
